from __future__ import unicode_literals, absolute_import

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext, ugettext_lazy as _

from filer.fields.image import FilerImageField
from cms.models.pluginmodel import CMSPlugin
from cms.models import Page
from cms.models.fields import PageField
from ckeditor.fields import RichTextField
#from cmsplugin_filer_image import ThumbnailOption

from . import fields
from djangocms_attributes_field.fields import AttributesField

class Classes(models.TextField):
    default_field_class = fields.Classes

    def __init__(self, *args, **kwargs):
        if 'verbose_name' not in kwargs:
            kwargs['verbose_name'] = 'Classes'
        if 'blank' not in kwargs:
            kwargs['blank'] = True
        if 'default' not in kwargs:
            kwargs['default'] = ''
        if 'help_text' not in kwargs:
            kwargs['help_text'] = 'Space separated classes that are added to ' + \
            	'the class. See <a href="http://getbootstrap.com/css/" ' + \
            	'target="_blank">Bootstrap 3 documentation</a>.'
        super(Classes, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Classes, self).formfield(**defaults)

SIZE_CHOICES = (
    ('100% auto', 'fit width: 100% auto'),
    ('auto 100%', 'fit height: auto 100%'),
    ('cover', 'fill: cover '),
    ('auto', 'default: auto'),
)

REPEAT_CHOICES = (
    ('no-repeat', 'no-repeat'),
    ('repeat', 'repeat'),
    ('repeat-x', 'repeat-x'),
    ('repeat-y', 'repeat-y'),
)

ATTACHMENT_CHOICES = (
    ('scroll', 'scroll'),
    ('fixed', 'fixed'),
)

# Should pull these from django settings.py
CONTAINER_CHOICES = (
    ('', 'No container (full-width)'),
    ('container', '.container (smaller content width based on device size)'),
    ('container-fluid', '.container-fluid (full-width with slight padding on sides)'),
)

@python_2_unicode_compatible  # only if you need to support Python 2
class Section(CMSPlugin):
    name = models.CharField('Section Name', max_length=25, default='', help_text='Descriptive name [not rendered on page]', blank=True, null=True)
    min_height = models.CharField('Minimum Section Height', max_length=25, default='0px', help_text='0 is default. Set it larger to expand height of section.')
    bg_image = FilerImageField(
        verbose_name=_('Background Image'),
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
    )
    bg_external_image = models.URLField(
        verbose_name=_('External URL Background Image'),
        blank=True,
        max_length=255,
        help_text=_('If provided, overrides the embedded image.')
    )
    bg_color = models.CharField('CSS Background Color', max_length=25, default='transparent', help_text='(e.g., #RRGGBB, rgba(120,120,120,0.3))')
    bg_size = models.CharField('Background Size', max_length=25, choices=SIZE_CHOICES, default='cover')
    bg_position = models.CharField('Background Position', max_length=25, default='center', blank=True)
    bg_repeat = models.CharField('Background Repeat', max_length=25, choices=REPEAT_CHOICES, default='no-repeat', blank=True)
    bg_attachment = models.CharField('Background Attachment', max_length=25, choices=ATTACHMENT_CHOICES, default='scroll', blank=True)
    container = models.CharField(
        verbose_name='Add nested .container or .container-fluid child element',
        max_length=25,
        choices=CONTAINER_CHOICES,
        blank=True,
        default='container',
        help_text=_('Adds a ".container" or ".container-fluid" element inside the section. '
            'Use it to have a full-page-width styled <section> with an inner container with '
            'a constrained width. All child plugins render inside the container.'),
    )

    classes = Classes()

    attributes = AttributesField(
        verbose_name='Attributes',
        blank=True,
        excluded_keys=['class'],
    )

    def __str__(self):
        return self.name + ' ' + self.container

    @property
    def bg_image_url(self):
        if self.bg_image:
            return self.bg_image.url
        else:
            return self.bg_external_image

    def clean(self):
        # you shall only set one image kind
        if self.bg_image and self.bg_external_image:
            raise ValidationError(
                ugettext('You need to add either an image or a URL '
                         'linking to an external image, not both.')
            )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-03-13 10:56
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import djangocms_attributes_field.fields
import djangocms_layouttools.models
import filer.fields.image


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0018_pagenode'),
        migrations.swappable_dependency(settings.FILER_IMAGE_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Section',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='djangocms_layouttools_section', serialize=False, to='cms.CMSPlugin')),
                ('name', models.CharField(blank=True, default='', help_text='Descriptive name [not rendered on page]', max_length=25, null=True, verbose_name='Section Name')),
                ('min_height', models.CharField(default='0px', help_text='0 is default. Set it larger to expand height of section.', max_length=25, verbose_name='Minimum Section Height')),
                ('bg_external_image', models.URLField(blank=True, help_text='If provided, overrides the embedded image.', max_length=255, verbose_name='External URL Background Image')),
                ('bg_color', models.CharField(default='transparent', help_text='(e.g., #RRGGBB, rgba(120,120,120,0.3))', max_length=25, verbose_name='CSS Background Color')),
                ('bg_size', models.CharField(choices=[('100% auto', 'fit width: 100% auto'), ('auto 100%', 'fit height: auto 100%'), ('cover', 'fill: cover '), ('auto', 'default: auto')], default='cover', max_length=25, verbose_name='Background Size')),
                ('bg_position', models.CharField(blank=True, default='center', max_length=25, verbose_name='Background Position')),
                ('bg_repeat', models.CharField(blank=True, choices=[('no-repeat', 'no-repeat'), ('repeat', 'repeat'), ('repeat-x', 'repeat-x'), ('repeat-y', 'repeat-y')], default='no-repeat', max_length=25, verbose_name='Background Repeat')),
                ('bg_attachment', models.CharField(blank=True, choices=[('scroll', 'scroll'), ('fixed', 'fixed')], default='scroll', max_length=25, verbose_name='Background Attachment')),
                ('container', models.CharField(blank=True, choices=[('', 'No container (full-width)'), ('container', '.container (smaller content width based on device size)'), ('container-fluid', '.container-fluid (full-width with slight padding on sides)')], default='container', help_text='Adds a ".container" or ".container-fluid" element inside the section. Use it to have a full-page-width styled <section> with an inner container with a constrained width. All child plugins render inside the container.', max_length=25, verbose_name='Add nested .container or .container-fluid child element')),
                ('classes', djangocms_layouttools.models.Classes(blank=True, default='', help_text='Space separated classes that are added to the class. See <a href="http://getbootstrap.com/css/" target="_blank">Bootstrap 3 documentation</a>.', verbose_name='Classes')),
                ('attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('bg_image', filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.FILER_IMAGE_MODEL, verbose_name='Background Image')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

import django.forms.fields
from django.forms.widgets import Textarea

class Classes(django.forms.fields.CharField):
    widget = django.forms.widgets.Textarea


# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_layout.models import Section


EXAMPLE_IMAGE = 'https://www.google.com/images/logo.png'


class SectionTestCase(TestCase):

    def setUp(self):
        Section.objects.create(
            min_height='400px',
            bg_color='#ff00ff',
            container='.container',
            bg_external_image=EXAMPLE_IMAGE,
        )

    def test_section_instance(self):
        """Section instance has been created"""
        section = Section.objects.get(container='.container')
        self.assertEqual(section.container, '.container')
        self.assertEqual(section.bg_image_url, EXAMPLE_IMAGE)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from django.utils.translation import ugettext_lazy as _

from cms.plugin_pool import plugin_pool
from cms.plugin_base import CMSPluginBase
from django.conf import settings

from .models import Section, CONTAINER_CHOICES

from django.contrib import admin
from django import forms
from django.contrib.admin.widgets import AdminRadioSelect
from django.contrib.admin.options import get_ul_class

# class SectionForm(forms.ModelForm):
#     # container = forms.ChoiceField(choices=CONTAINER_CHOICES, widget=forms.RadioSelect)
#     class Meta:
#         model = Section
#         widgets = {
#            'container': AdminRadioSelect(attrs={'class': get_ul_class(admin.HORIZONTAL),}),
#         }
#         # fields = ['container']
#         fields = '__all__'

class SectionPlugin(CMSPluginBase):
	model = Section
	radio_fields = { 'container': admin.VERTICAL }
	# form = SectionForm
	name = _("Page Section")
	module = 'Advanced Layout'
	render_template = "djangocms_layouttools/section.html"
	allow_children = True
	text_enabled = False
	#raw_id_fields = ('group',)
	admin_preview = False

plugin_pool.register_plugin(SectionPlugin)


from django.contrib import admin
from django import forms
from django.contrib.admin.widgets import AdminRadioSelect

from .models import Section, CONTAINER_CHOICES

'''
class SectionForm(forms.ModelForm):
    # container = forms.ChoiceField(choices=CONTAINER_CHOICES, widget=forms.RadioSelect)
    class Meta:
        model = Section
        widgets = {
           'container': forms.RadioSelect,
        }
        # fields = ['container']
        fields = '__all__'

class SectionAdmin(admin.ModelAdmin):
    model = Section
    radio_fields = { 'container': admin.VERTICAL }

admin.site.register(Section, SectionAdmin)
'''